"""
The qiprofile-rest unit tests.

:Note: These tests require an active qiprofile REST server.
  The tests create and delete a ``qiprofile_test`` database.
  A more thorough model test is provided by the ``qiprofile-rest``
  server project's ``test_seed.py`` unit test.
"""
